<?php

/*
 +-----------------------------------------------------------------------+
 | lib/drivers/ldap.php                                                  |
 |                                                                       |
 | Copyright (C) 2009 Boris HUISGEN <bhuisgen@hbis.fr>                   |
 | Licensed under the GNU GPL                                            |
 | Modified by Mihai Badici <mbadici@gmail.com>                          |
 +-----------------------------------------------------------------------+
 */

/*
 * Read driver function.
 *
 * @param array $data the array of data to get and set.
 *
 * @return boolean TRUE if load is successfull; FALSE otherwise.
 */
function vacation_read(array &$data)
{
//	require_once ('Net/LDAP2.php');
	$rcmail = rcmail::get_instance();
	
	$search = array('%username',
					'%email_local',
					'%email_domain',
					'%email');
	$replace = array($data['username'],
					 $data['email_local'],
					 $data['email_domain'],
					 $data['email']);	
	$ldap_basedn = str_replace($entry, $replace, $rcmail->config->get('vacation_ldap_basedn'));
	$ldap_binddn = str_replace($entry, $replace, $rcmail->config->get('vacation_ldap_binddn'));

	$search = array('%username',
					'%password',
					'%email_local',
					'%email_domain',
					'%email');
	$replace = array($data['username'],
					 $rcmail->decrypt($_SESSION['password']),
					 $data['email_local'],
					 $data['email_domain'],
					 $data['email']);
	$ldap_bindpw = str_replace($entry, $replace, $rcmail->config->get('vacation_ldap_bindpw'));
	
	$ldapConfig = array (
        'host'      => $rcmail->config->get('vacation_ldap_host'),
        'port'      => $rcmail->config->get('vacation_ldap_port'),
        'starttls'  => $rcmail->config->get('vacation_ldap_starttls'),
        'version'   => $rcmail->config->get('vacation_ldap_version'),
        'basedn'    => $ldap_basedn,
        'binddn'    => $ldap_binddn,
        'bindpw'    => $ldap_bindpw,
	);

	if(!($ldap = ldap_connect($ldapConfig['host'])))

	{
		return PLUGIN_ERROR_CONNECT;
	}

	$search = array('%username',
					'%email_local',
					'%email_domain',
					'%email',
					'%vacation_enable',
					'%vacation_start',
					'%vacation_end',
					'%vacation_subject',
					'%vacation_message',
					'%vacation_keepcopyininbox',
					'%vacation_forwarder');
	$replace = array($data['username'],
					 $data['email_local'],
					 $data['email_domain'],
					 $data['email'],
					 $data['vacation_enable'],
					 $data['vacation_start'],
					 $data['vacation_end'],
					 $data['vacation_subject'],
					 $data['vacation_message'],
					 $data['vacation_keepcopyininbox'],
					 $data['vacation_forwarder']);

	$search_base = str_replace($search, $replace, $rcmail->config->get('vacation_ldap_search_base'));

	
	$search_filter = str_replace($search, $replace, $rcmail->config->get('vacation_ldap_search_filter'));
	$attrs = $rcmail->config->get('vacation_ldap_search_attrs');

	$res = ldap_search($ldap,$search_base, $search_filter, $attrs);
	if ($res== NULL)
	{
		ldap_close($ldap);

		return PLUGIN_ERROR_PROCESS;
	}

	if (!($entry=ldap_first_entry($ldap, $res)))
	{
		ldap_close($ldap);

		return PLUGIN_ERROR_PROCESS;
	}
	
	    $val=ldap_get_values($ldap,$entry,$rcmail->config->get('vacation_ldap_attr_vacationenable'));
	{
		if ($val[0] == $rcmail->config->get('vacation_ldap_attr_vacationenable_value_enabled'))
			$data['vacation_enable'] = 1;
		else
			$data['vacation_enable'] = 0;
	}
	$val=ldap_get_values($ldap,$entry,$rcmail->config->get('vacation_ldap_attr_vacationstart'));
	$data['vacation_start'] = $val[0];
	$val= ldap_get_values($ldap,$entry,$rcmail->config->get('vacation_ldap_attr_vacationend'));
		$data['vacation_end'] = $val[0];
		$val =ldap_get_values($ldap,$entry,$rcmail->config->get('vacation_ldap_attr_vacationsubject'));
		$data['vacation_subject'] = $val[0];
		$val=ldap_get_values($ldap,$entry,$rcmail->config->get('vacation_ldap_attr_vacationmessage'));
		$data['vacation_message'] = $val[0];
//			$data['vacation_keepcopyininbox'] = 1;
//			$data['vacation_keepcopyininbox'] = 0;
		$val= ldap_get_values($ldap,$entry,$rcmail->config->get('vacation_ldap_attr_vacation_forwarder'));
		
		$data['vacation_forwarder'] = $val[0];

    ldap_close($ldap);

	return PLUGIN_SUCCESS;
}

/*
 * Write driver function.
 *
 * @param array $data the array of data to get and set.
 *
 * @return boolean TRUE if save is successfull; FALSE otherwise.
 */
function vacation_write(array &$data)
{
	$rcmail = rcmail::get_instance();
	
	$search = array('%username',
					'%email_local',
					'%email_domain',
					'%email');
	$replace = array($data['username'],
					 $data['email_local'],
					 $data['email_domain'],
					 $data['email']);	
	$ldap_basedn = str_replace($search, $replace, $rcmail->config->get('vacation_ldap_basedn'));
	$ldap_binddn = str_replace($search, $replace, $rcmail->config->get('vacation_ldap_binddn'));

	$search = array('%username',
					'%password',
					'%email_local',
					'%email_domain',
					'%email');
	$replace = array($data['username'],
					 $rcmail->decrypt($_SESSION['password']),
					 $data['email_local'],
					 $data['email_domain'],
					 $data['email']);
	$ldap_bindpw = str_replace($search, $replace, $rcmail->config->get('vacation_ldap_bindpw'));
	
	$ldapConfig = array (
        'host'      => $rcmail->config->get('vacation_ldap_host'),
        'port'      => $rcmail->config->get('vacation_ldap_port'),
        'starttls'  => $rcmail->config->get('vacation_ldap_starttls'),
        'version'   => $rcmail->config->get('vacation_ldap_version'),
        'basedn'    => $ldap_basedn,
        'binddn'    => $ldap_binddn,
        'bindpw'    => $ldap_bindpw,
	);

	if(!($ldap = ldap_connect($ldapConfig['host'])))

	{
	return PLUGIN_ERROR_CONNECT;
	}

	$search_base = str_replace($search, $replace, $rcmail->config->get('vacation_ldap_search_base'));
	$search_filter = str_replace($search, $replace, $rcmail->config->get('vacation_ldap_search_filter'));
	$attrs  = array($rcmail->config->get('vacation_ldap_attr_vacationenable'), $rcmail->config->get('vacation_ldap_attr_vacationmessage'), $rcmail->config->get('vacation_ldap_attr_vacationstart'), $rcmail->config->get('vacation_ldap_attr_vacationend'));
	$res = ldap_search($ldap,$search_base, $search_filter, $attrs);
	$entry=ldap_first_entry($ldap, $res);
	$dn = ldap_get_dn($ldap,$entry);
	
	ldap_set_option($ldap,LDAP_OPT_PROTOCOL_VERSION,3);
	ldap_set_option($ldap,LDAP_OPT_REFERRALS,0);

	
    ldap_bind($ldap,$dn,$rcmail->decrypt($_SESSION['password']));

	

/*	$dns = $rcmail->config->get('vacation_ldap_modify_dns');
	$ops = $rcmail->config->get('vacation_ldap_modify_ops');
	
	for ($i = 0; $i < count($dns) && $i < count($ops); $i++)
	{
		$search = array('%username',
						'%email_local',
						'%email_domain',
						'%email',
						'%vacation_enable',
						'%vacation_start',
						'%vacation_end',
						'%vacation_subject',
						'%vacation_message',
						'%vacation_keepcopyininbox',
						'%vacation_forwarder',
		);
		$replace = array($data['username'],
						 $data['email_local'],
						 $data['email_domain'],
						 $data['email'],
						 ($data['vacation_enable'] ? "TRUE" : "FALSE"),
						 $data['vacation_start'],
						 $data['vacation_end'],
						 $data['vacation_subject'],
						 $data['vacation_message'],
						 $data['vacation_keepcopyininbox'],
						 $data['vacation_forwarder']
		);
		$dns[$i] = str_replace($search, $replace, $dns[$i]);

//		$ret = ldap_mod_replace($ldap,$dn, $replace);

		foreach ($ops[$i] as $op => $args)
		{
			foreach ($args as $key => $value)
			{
				$search = array('%username',
								'%email_local',
								'%email_domain',
								'%email',
								'%vacation_enable',
								'%vacation_start',
								'%vacation_end',
								'%vacation_subject',
								'%vacation_message',
								'%vacation_keepcopyininbox',
								'%vacation_forwarder'
				);
				$replace = array($data['username'],
								 $data['email_local'],
								 $data['email_domain'],
								 $data['email'],
								($data['vacation_enable'] ?
									$rcmail->config->get('vacation_ldap_attr_vacationenable_value_enabled') :
									$rcmail->config->get('vacation_ldap_attr_vacationenable_value_disabled')),
								$data['vacation_start'],
								$data['vacation_end'],
								$data['vacation_subject'],
								$data['vacation_message'],
								$data['vacation_keepcopyininbox'],
								$data['vacation_forwarder']
				);
//				$ops[$i][$op][$key] = str_replace($search, $replace, $value);				
			}
		}
*/

$modify[$rcmail->config->get('vacation_ldap_attr_vacationenable')]= ($data['vacation_enable'] ?
									$rcmail->config->get('vacation_ldap_attr_vacationenable_value_enabled') :
									$rcmail->config->get('vacation_ldap_attr_vacationenable_value_disabled'))
	;

$modify[$rcmail->config->get('vacation_ldap_attr_vacationmessage')] = $data['vacation_message'];
$modify[$rcmail->config->get('vacation_ldap_attr_vacationstart')] =$data['vacation_start'];
$modify[$rcmail->config->get('vacation_ldap_attr_vacationend')] =$data['vacation_end'];
$modify[$rcmail->config->get('vacation_ldap_attr_vacation_forwarder')] =$data['vacation_forwarder'];
//$modify[$rcmail->config->get('vacation_ldap_attr_vacationsubject')] =$data['vacation_subject'];
							

		$ret = ldap_mod_replace($ldap,$dn, $modify);
		if(!$ret)
		{
			ldap_close($ldap);
			
			return PLUGIN_ERROR_PROCESS;
		}


	ldap_close($ldap);

	return PLUGIN_SUCCESS;
}
